@@ -28,11 +28,12 @@ from dj_rql.constants import (
 from dj_rql.exceptions import RQLFilterLookupError, RQLFilterParsingError, RQLFilterValueError
 from dj_rql.openapi import RQLFilterClassSpecification
 from dj_rql.parser import RQLParser
-from dj_rql.qs import Annotation
+from dj_rql.qs import Annotation, NPR, NSR
 from dj_rql.transformer import RQLToDjangoORMTransformer
 
-from django.db.models import Model, Q
+from django.db.models import ForeignKey, ManyToManyField, Model, OneToOneField, OneToOneRel, Q
 from django.utils.dateparse import parse_date, parse_datetime
+from django.utils.functional import cached_property
 
 from lark.exceptions import LarkError
 
@@ -1040,8 +1041,7 @@ class AutoRQLFilterClass(RQLFilterClass):
     """This class will collect all simple model fields except the ones in this field."""
 
     def _get_init_filters(self):
-        described_filters = tuple(self.FILTERS) if self.FILTERS else ()
-
+        described_filters = self._described_filters
         filters = tuple(
             {
                 'filter': f.name,
@@ -1057,3 +1057,111 @@ class AutoRQLFilterClass(RQLFilterClass):
         )
 
         return described_filters + filters
+
+    @cached_property
+    def _described_filters(self):
+        return tuple(self.FILTERS) if self.FILTERS else ()
+
+
+class NestedAutoRQLFilterClass(AutoRQLFilterClass):
+    """
+    Filter class that automatically collects filters for all model fields with
+    specified depth for related models.
+    """
+    SELECT = True
+
+    DEPTH = 1
+    """
+    Specifies how deep model relations will be traversed.
+    If `DEPTH = 0` this class behaves as `AutoRQLFilterClass`.
+    """
+
+    def _get_init_filters(self):
+        if self.DEPTH == 0:
+            return super()._get_init_filters()
+
+        depth = 0
+        global_namespace = []
+        iterator = [(self.MODEL, global_namespace, None, None)]
+
+        while depth <= self.DEPTH and iterator:
+            iterator = self._iter_models_to_get_filters(depth, iterator)
+            depth += 1
+
+        return self._described_filters + tuple(global_namespace)
+
+    def _iter_models_to_get_filters(self, depth, iterator):
+        related_models = []
+
+        for model_data in iterator:
+            related_models.extend(self._iter_model_to_get_filters(depth, model_data))
+
+        return related_models
+
+    def _iter_model_to_get_filters(self, depth, model_data):
+        model, namespace, circular_related_name, prefix = model_data
+        through_models = set()
+        model_related_models = []
+
+        for field in model._meta.get_fields():
+            rel_f_name = self._get_relative_field_name(field, circular_related_name, prefix)
+            if not rel_f_name:
+                continue
+
+            if field.is_relation:
+                if self._is_through_field(field):
+                    through_models.add(field.through)
+
+                relation_data = self._add_relation_to_iterated_models(depth, field, namespace)
+                model_related_models.append(relation_data + (rel_f_name,))
+                continue
+
+            namespace.append({
+                'filter': field.name,
+                'ordering': True,
+                'search': FilterTypes.field_filter_type(field) == FilterTypes.STRING,
+            })
+
+        return [i for i in model_related_models if i[0] not in through_models]
+
+    def _add_relation_to_iterated_models(self, depth, field, namespace):
+        if isinstance(field, (ForeignKey, ManyToManyField)):
+            circular_related_name = field.remote_field.name
+        else:
+            circular_related_name = field.field.name
+
+        namespace_filters = []
+        if depth < self.DEPTH:
+            namespace.append({
+                'namespace': field.name,
+                'filters': namespace_filters,
+                'qs': self._get_field_optimization(field),
+            })
+
+        return field.related_model, namespace_filters, circular_related_name
+
+    def _get_relative_field_name(self, field, circular_related_name, prefix):
+        field_name = field.name
+        if circular_related_name and field_name == circular_related_name:
+            # This is needed to avoid circular dependencies
+            return
+
+        rel_f_name = '.'.join((prefix, field_name)) if prefix else field_name
+        if rel_f_name in self.EXCLUDE_FILTERS or rel_f_name in self._described_filters:
+            return
+
+        return rel_f_name
+
+    def _get_field_optimization(self, field):
+        if not self.SELECT:
+            return
+
+        if isinstance(field, (ForeignKey, OneToOneField, OneToOneRel)):
+            return NSR(field.name)
+
+        if not self._is_through_field(field):
+            return NPR(field.name)
+
+    @staticmethod
+    def _is_through_field(field):
+        return getattr(field, 'through', None)
